# Copyright (c) 2012, Intel Corporation
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#     * Redistributions of source code must retain the above copyright notice,
#       this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright notice,
#       this list of conditions and the following disclaimer in the documentation
#       and/or other materials provided with the distribution.
#     * Neither the name of Intel Corporation nor the names of its contributors
#       may be used to endorse or promote products derived from this software
#       without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
# ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""Tests for ACPI"""

import acpi
import struct
import testsuite

def register_tests():
    testsuite.add_test("ACPI _MAT (Multiple APIC Table Entry) under Processor objects", test_mat, submenu="ACPI Tests")

def test_mat():
    cpupaths = acpi.get_cpupaths()
    procid_apicid, uid_x2apicid = acpi.parse_madt()
    for cpupath in cpupaths:
        # Find the ProcId defined by the processor object
        processor = acpi.evaluate(cpupath)
        # Find the UID defined by the processor object's _UID method
        uid = acpi.evaluate(cpupath + "._UID")
        mat_buffer = acpi.evaluate(cpupath + "._MAT")
        if mat_buffer is None:
            continue
        # Process each _MAT subtable
        i = 0
        while i != len(mat_buffer):
            subtable = acpi.SubTable._make(struct.unpack_from(acpi.SubTable.fmt_str, mat_buffer, i))
            if subtable.Type == acpi.MADT_TYPE_LOCAL_APIC:
                localApic = acpi.MadtLocalApic._make(struct.unpack_from(acpi.MadtLocalApic.fmt_str, mat_buffer, i))
                if localApic.Flags & acpi.MADT_ENABLED:
                    if testsuite.test("{} Processor declaration ProcId = _MAT ProcId".format(cpupath), processor.ProcId == localApic.ProcId):
                        testsuite.print_detail("{} ProcId ({:#02x}) != _MAT ProcId ({:#02x})".format(cpupath, processor.ProcId, localApic.ProcId))
                        testsuite.print_detail("Processor Declaration: {}".format(processor))
                        testsuite.print_detail("_MAT offset[{}]: {}".format(i, localApic))
                    if testsuite.test("{} with local APIC in _MAT has local APIC in MADT".format(cpupath), processor.ProcId in procid_apicid):
                        pass
                    elif testsuite.test("{} ApicId derived using Processor declaration ProcId = _MAT ApicId".format(cpupath), procid_apicid[processor.ProcId] == localApic.ApicId):
                        testsuite.print_detail("{} ApicId derived from MADT ({:#02x}) != _MAT ApicId ({:#02x})".format(cpupath, procid_apicid[processor.ProcId], localApic.ApicId))
                        testsuite.print_detail("Processor Declaration: {}".format(processor))
                        testsuite.print_detail("_MAT offset[{}]: {}".format(i, localApic))
            if subtable.Type == acpi.MADT_TYPE_LOCAL_X2APIC:
                x2Apic = acpi.MadtLocalx2Apic._make(struct.unpack_from(acpi.MadtLocalx2Apic.fmt_str, mat_buffer, i))
                if x2Apic.Flags & acpi.MADT_ENABLED:
                    if testsuite.test("{} with x2Apic in _MAT has _UID".format(cpupath), uid is not None):
                        pass
                    elif testsuite.test("{}._UID = _MAT UID".format(cpupath), uid == x2Apic.UID):
                        testsuite.print_detail("{}._UID ({:#x}) != _MAT UID ({:#x})".format(cpupath, uid, x2Apic.UID))
                        testsuite.print_detail("_MAT offset[{}]: {}".format(i, x2Apic))
                    if testsuite.test("{} with _MAT x2Apic has x2Apic in MADT".format(cpupath), x2Apic.UID in uid_x2apicid):
                        pass
                    elif testsuite.test("{} x2ApicId derived from MADT using UID = _MAT x2ApicId".format(cpupath), uid_x2apicid[x2Apic.UID] == x2Apic.X2ApicId):
                        testsuite.print_detail("{} x2ApicId derived from MADT ({:#02x}) != _MAT x2ApicId ({:#02x})".format(cpupath, uid_x2apicid[x2Apic.UID], x2Apic.X2ApicId))
                        testsuite.print_detail("_MAT offset[{}]: {}".format(i, x2Apic))
            i += subtable.Length
